from ...common import DESCRIPTION
from ...common import VERSION
import argparse
import sys

# SkeleParser is a wrapper around the parsing object that allows it to possbily be replaced in the future with custom logic
class SkeleParser:

    parser = None
    config = None
    env = None
    desc = None

    # Initialize the parser with the given config and environment
    def __init__(self, config=None, env=None):
        self.config = config
        self.env = env
        self.desc = self.buildDescription()

        # ---Standard Parser Setup---

        # Construct the root argument parser from which all sub-parsers will be built
        self.parser = argparse.ArgumentParser(description=self.desc, formatter_class=argparse.RawTextHelpFormatter)
        subparsers = self.parser.add_subparsers(dest="job")

        if (config.name is None):
            # Add SCAFFOLD parser
            scaffoldParser = subparsers.add_parser("scaffold", help="Scaffold a new or existing project with Skelebot")
            scaffoldParser.add_argument("-e", "--existing", action='store_true', help="Scaffold an existing project without creating new folder")
        else:
            # Add STANDARD PARAMS
            self.parser.add_argument("-e", "--env", help="Specify the runtime environment configurations")
            self.parser.add_argument("-s", "--skip-build", action='store_true', help="Skip the build process and attempt to use previous docker build")
            self.parser.add_argument("-n", "--native", action='store_true', help="Run natively instead of through Docker")

        # ---Config Based Parser Setup---

        # Add JOBS
        if (config.jobs != None):
            for job in config.jobs:
                subparser = subparsers.add_parser(job.name, help=job.help + " (" + job.source + ")")

                # Add ARGS and PARAMS
                subparser = self.addArgs(job.args, subparser)
                subparser = self.addParams(job.params, subparser)
                subparser = self.addParams(config.params, subparser)

        # Add COMPONENT parsers
        for component in self.config.components:
            subparsers = component.addParsers(subparsers)

    # Parse the args from the parser built by the config
    def parseArgs(self, args=sys.argv[1:]):
        return self.parser.parse_args(args)

    # Display the help message from the internal parser object
    def showHelp(self):
        return self.parser.print_help()

    # Add args to the given subparser
    def addArgs(self, args, subparser):
        if (args != None):
            for arg in args:
                if not arg.choices:
                    subparser.add_argument(arg.name, help=arg.help)
                else:
                    subparser.add_argument(arg.name, choices=arg.choices, help=arg.help)
        return subparser

    # Add params to the given subparser
    def addParams(self, params, subparser):
        if (params != None):
            for param in params:
                flags = ["--{}".format(param.name)]
                if (param.alt is not None):
                    flags.append("-{}".format(param.alt))

                if param.choices:
                    subparser.add_argument(*flags, choices=param.choices, default=param.default, help=param.help)
                elif param.accepts == "boolean":
                    subparser.add_argument(*flags, action='store_true', help=param.help)
                elif param.accepts == "list":
                    subparser.add_argument(*flags, nargs='*', default=param.default, help=param.help)
                else:
                    subparser.add_argument(*flags, default=param.default, help=param.help)
        return subparser

    # Construct the description text for the '--help' output
    def buildDescription(self):
        description = "Skelebot Version: {version}".format(version=VERSION)
        if (self.config.name != None):
            name = " ".join([word.capitalize() for word in self.config.name.split("-")])
            description = self.config.description
            version = self.config.version
            description = DESCRIPTION.format(version=VERSION, project=name, desc=description, pVersion=version, env=self.env)

        return description



from .skeleYaml import SkeleYaml
from .param import Param
from .arg import Arg

class Job(SkeleYaml):
    name = None
    source = None
    mode = None
    help = None
    args = None
    params = None
    ignores = None
    mappings = None

    def __init__(self, name=None, source=None, mode=None, help=None, args=[], params=[], ignores=[], mappings=[]):
        self.name = name
        self.source = source
        self.mode = "i" if mode == None else mode
        self.help = help
        self.args = args
        self.params = params
        self.ignores = ignores
        self.mappings = mappings

    @classmethod
    def load(cls, config):
        values = {}
        for attr in config.keys():
            if (attr in list(vars(Job).keys())):
                if (attr == "args"):
                    values[attr] = Arg.loadList(config[attr])
                elif (attr == "params"):
                    values[attr] = Param.loadList(config[attr])
                else:
                    values[attr] = config[attr]

        return cls(**values)

    @classmethod
    def loadList(cls, config):
        jobs = []
        for element in config:
            jobs.append(cls.load(element))

        return jobs



from ..objects.component import *
from ..systems.execution import docker

# This component provides the ability to spin up Jupyter in Docker for any project
class Jupyter(Component):
    activation = Activation.PROJECT
    commands = ["jupyter"]

    port = None
    folder = None

    # Allows for intelligent defaults in the constructor so projects without config can still use the component
    def __init__(self, port=8888, folder="."):
        self.port = port
        self.folder = folder

    # Parser for the command that spins up Jupyter inside the Docker Container based on the given port and folder
    def addParsers(self, subparsers):
        helpMessage = "Spin up Jupyter in a Docker Container (port = {port}, folder = {folder})".format(port=self.port, folder=self.folder)
        parser = subparsers.add_parser("jupyter", help=helpMessage)
        return subparsers

    # Build the docker image and then run the container with the Jupyter command, port mapped, and folder volume mapped
    def execute(self, config, args):

        docker.build(config)

        command = "jupyter notebook --ip=0.0.0.0 --port=8888 --allow-root --notebook-dir={folder}".format(folder=self.folder)
        ports = ["{port}:8888".format(port=self.port)]

        print("Notebook Starting on localhost:{port}".format(port=self.port))
        print("Copy the token below to authenticate with Jupyter")

        return docker.run(config, command, "i", ports, ".", "jupyter")



from unittest import TestCase
from unittest import mock

import skelebot as sb
import os

class TestDockerfile(TestCase):
    path = ""

    # Get the path to the current working directory before we mock the function to do so
    def setUp(self):
        self.path = os.getcwd()

    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    def test_buildDockerfile_no_language(self, mock_getcwd, mock_expanduser):
        folderPath = "{path}/test/files".format(path=self.path)
        filePath = "{folder}/Dockerfile".format(folder=folderPath)
        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath
        config = sb.systems.generators.yaml.loadConfig()
        config.language = None

        expectedDockerfile = """# This Dockerfile was generated by Skelebot
# Editing this file manually is not advised as all changes will be overwritten during Skelebot execution
FROM ubuntu:18.04
MAINTAINER Mega Man <megaman@cars.com>
WORKDIR /app
COPY . /app
CMD /bin/bash -c './/app/build.sh'\n"""

        sb.systems.generators.dockerfile.buildDockerfile(config)

        data = None
        with open(filePath, "r") as file:
            data = file.read()
        self.assertTrue(data is not None)
        self.assertEqual(data, expectedDockerfile)

    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    def test_buildDockerfile_base(self, mock_getcwd, mock_expanduser):
        folderPath = "{path}/test/files".format(path=self.path)
        filePath = "{folder}/Dockerfile".format(folder=folderPath)

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath
        config = sb.systems.generators.yaml.loadConfig()
        config.language = "R"
        config.dependencies.append("github:github.com/repo:cool-lib")
        config.dependencies.append("file:libs/proj:cool-proj")
        config.dependencies.append("dtable=9.0")

        expectedDockerfile = """# This Dockerfile was generated by Skelebot
# Editing this file manually is not advised as all changes will be overwritten during Skelebot execution
FROM skelebot/r-base
MAINTAINER Mega Man <megaman@cars.com>
WORKDIR /app
RUN ["Rscript", "-e", "install.packages('pyyaml', repo='https://cloud.r-project.org'); library(pyyaml)"]
RUN ["Rscript", "-e", "install.packages('artifactory', repo='https://cloud.r-project.org'); library(artifactory)"]
RUN ["Rscript", "-e", "install.packages('argparse', repo='https://cloud.r-project.org'); library(argparse)"]
RUN ["Rscript", "-e", "install.packages('coverage', repo='https://cloud.r-project.org'); library(coverage)"]
RUN ["Rscript", "-e", "install.packages('pytest', repo='https://cloud.r-project.org'); library(pytest)"]
RUN ["Rscript", "-e", "library(devtools); install_github('github.com/repo'); library(cool-lib)"]
COPY libs/proj libs/proj
RUN ["Rscript", "-e", "install.packages('/app/libs/proj', repos=NULL, type='source'); library(cool-proj)"]
RUN ["Rscript", "-e", "library(devtools); install_version('dtable', version='9.0', repos='http://cran.us.r-project.org'); library(dtable)"]
COPY . /app
CMD /bin/bash -c './/app/build.sh'\n"""

        sb.systems.generators.dockerfile.buildDockerfile(config)

        data = None
        with open(filePath, "r") as file:
            data = file.read()
        self.assertTrue(data is not None)
        print(data)
        self.assertEqual(data, expectedDockerfile)

    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    def test_buildDockerfile_krb(self, mock_getcwd, mock_expanduser):
        folderPath = "{path}/test/files".format(path=self.path)
        filePath = "{folder}/Dockerfile".format(folder=folderPath)

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath
        config = sb.systems.generators.yaml.loadConfig()
        config.language = "R"
        config.dependencies.append("github:github.com/repo:cool-lib")
        config.dependencies.append("file:libs/proj:cool-proj")
        config.dependencies.append("dtable=9.0")
        config.components.append(sb.components.kerberos.Kerberos())

        expectedDockerfile = """# This Dockerfile was generated by Skelebot
# Editing this file manually is not advised as all changes will be overwritten during Skelebot execution
FROM skelebot/r-krb
MAINTAINER Mega Man <megaman@cars.com>
WORKDIR /app
RUN ["Rscript", "-e", "install.packages('pyyaml', repo='https://cloud.r-project.org'); library(pyyaml)"]
RUN ["Rscript", "-e", "install.packages('artifactory', repo='https://cloud.r-project.org'); library(artifactory)"]
RUN ["Rscript", "-e", "install.packages('argparse', repo='https://cloud.r-project.org'); library(argparse)"]
RUN ["Rscript", "-e", "install.packages('coverage', repo='https://cloud.r-project.org'); library(coverage)"]
RUN ["Rscript", "-e", "install.packages('pytest', repo='https://cloud.r-project.org'); library(pytest)"]
RUN ["Rscript", "-e", "library(devtools); install_github('github.com/repo'); library(cool-lib)"]
COPY libs/proj libs/proj
RUN ["Rscript", "-e", "install.packages('/app/libs/proj', repos=NULL, type='source'); library(cool-proj)"]
RUN ["Rscript", "-e", "library(devtools); install_version('dtable', version='9.0', repos='http://cran.us.r-project.org'); library(dtable)"]
COPY . /app
CMD /bin/bash -c './/app/build.sh'\n"""

        sb.systems.generators.dockerfile.buildDockerfile(config)

        data = None
        with open(filePath, "r") as file:
            data = file.read()
        self.assertTrue(data is not None)
        print(data)
        self.assertEqual(data, expectedDockerfile)

    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    def test_buildDockerfile_custom(self, mock_getcwd, mock_expanduser):
        folderPath = "{path}/test/files".format(path=self.path)
        filePath = "{folder}/Dockerfile".format(folder=folderPath)

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath
        config = sb.systems.generators.yaml.loadConfig()
        config.baseImage = "whatever:uwant"

        expectedDockerfile = """# This Dockerfile was generated by Skelebot
# Editing this file manually is not advised as all changes will be overwritten during Skelebot execution
FROM whatever:uwant
MAINTAINER Mega Man <megaman@cars.com>
WORKDIR /app
RUN ["pip", "install", "pyyaml"]
RUN ["pip", "install", "artifactory"]
RUN ["pip", "install", "argparse"]
RUN ["pip", "install", "coverage"]
RUN ["pip", "install", "pytest"]
COPY . /app
CMD /bin/bash -c './/app/build.sh'\n"""

        sb.systems.generators.dockerfile.buildDockerfile(config)

        data = None
        with open(filePath, "r") as file:
            data = file.read()
        self.assertTrue(data is not None)
        print(data)
        self.assertEqual(data, expectedDockerfile)

if __name__ == '__main__':
    unittest.main()



from ..objects.component import Activation
from ..common import SKELEBOT_HOME, PLUGINS_HOME
from .plugin import Plugin
from .jupyter import Jupyter
from .kerberos import Kerberos
from .bump import Bump
from .prime import Prime
from .dexec import Dexec
from .artifactory import Artifactory

import os
import sys
import importlib

class ComponentFactory():

    COMPONENTS = None

    def __init__(self):
        self.COMPONENTS = {
            Plugin.__name__.lower(): Plugin,
            Jupyter.__name__.lower(): Jupyter,
            Kerberos.__name__.lower(): Kerberos,
            Bump.__name__.lower(): Bump,
            Prime.__name__.lower(): Prime,
            Dexec.__name__.lower(): Dexec,
            Artifactory.__name__.lower(): Artifactory
        }

        # Add the plugin components to the master list
        pluginsHome = os.path.expanduser(PLUGINS_HOME)
        if (os.path.exists(pluginsHome)):
            sys.path.append(pluginsHome)
            for pluginName in os.listdir(pluginsHome):
                if (pluginName[0] != "_"):
                    module = importlib.import_module("{name}.{name}".format(name=pluginName))
                    plugin = getattr(module, pluginName[0].upper() + pluginName[1:])
                    self.COMPONENTS[pluginName.lower()] = plugin

    def buildComponents(self, activations=[], ignores=[]):
        components = []
        for component in list(self.COMPONENTS.values()):
            if (component.activation in activations) and (component.__name__ not in ignores):
                components.append(component())

        return components

    def buildComponent(self, name, data=None):
        return self.COMPONENTS[name].load(data) if (name in self.COMPONENTS) else None



def promptUser(message, options=None, boolean=False):
    inp = None

    if (options):
        options = list(map(str, options))
        while inp not in options:
            inp = input("{msg} [{options}]: ".format(msg=message, options= ', '.join(options)))
    elif (boolean):
        inp = input("{msg} [Y/N]: ".format(msg=message)) in ["Y", "y"]
    else:
        inp = input("{msg}: ".format(msg=message))

    return inp



from unittest import TestCase
from unittest import mock

import skelebot as sb
import argparse

class TestPrime(TestCase):

    def test_addParsers(self):
        parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter)
        subparsers = parser.add_subparsers(dest="prime")
        prime = sb.components.prime.Prime()
        subparsers = prime.addParsers(subparsers)

        self.assertNotEqual(subparsers.choices["prime"], None)

    @mock.patch('skelebot.components.prime.docker')
    def test_execute(self, mock_docker):
        config = sb.objects.config.Config()
        args = argparse.Namespace()

        prime = sb.components.prime.Prime()
        prime.execute(config, args)

        mock_docker.build.assert_called_with(config)

if __name__ == '__main__':
    unittest.main()



from .skeleYaml import SkeleYaml
from enum import Enum

class Activation(Enum):
    EMPTY = 1
    CONFIG = 2
    PROJECT = 3
    ALWAYS = 4

class Component(SkeleYaml):
    activation = Activation.CONFIG
    commands = []

    @classmethod
    def load(cls, config): return cls(**config)

    # Scaffolding System Hooks
    def scaffold(self): return None

    # Parsing System Hooks
    def addParsers(self, subparsers): return subparsers

    # Generator System Hooks
    def appendDockerignore(self): return ""
    def appendDockerfile(self): return ""

    # Execution System Hooks
    def execute(self, config, args): return None
    def prependCommand(self, job, native): return None
    def appendCommand(self, job, native): return None
    def addDockerRunParams(self): return None



from ...objects.config import Config
from ...systems.generators import dockerfile
from ...systems.generators import dockerignore

import os

BUILD_CMD = "docker build -t {image} ."
RUN_CMD = "docker run --name {image}-{jobName} --rm {params} {image} /bin/bash -c '{command}'"

def build(config):
    dockerfile.buildDockerfile(config)
    dockerignore.buildDockerignore(config)

    status = os.system(BUILD_CMD.format(image=config.getImageName()))
    if (config.ephemeral == True):
        os.remove("Dockerfile")
        os.remove(".dockerignore")

    if (status > 0):
        raise Exception("Docker Build Failed")

    return status

def run(config, command, mode, ports, mappings, taskName):
    params = "-{mode}".format(mode=mode)

    # Construct the port mappings
    if (ports):
        for port in ports:
            params += " -p {port}".format(port=port)

    # Construct the volume mappings
    if (mappings):
        for vmap in mappings:
            if ("~" in vmap):
                vmap = vmap.replace("~", os.path.expanduser("~"))

            if (":" in vmap):
                params += " -v {vmap}".format(vmap=vmap)
            else:
                params += " -v $PWD/{vmap}:/app/{vmap}".format(vmap=vmap)

    # Construct the additional parameters from the components
    for component in config.components:
        addParams = component.addDockerRunParams()
        if (addParams is not None):
            params += " {params}".format(params=addParams)

    # Assuming the image was built without errors, run the container with the given args, params, and config
    return os.system(RUN_CMD.format(image=config.getImageName(), jobName=taskName, command=command, params=params, mode=mode))



import copy

class SkeleYaml():

    def toDict(self):
        dct = copy.deepcopy(vars(self))
        removeList = []
        for key in dct.keys():
            if (dct[key] is None) or (dct[key] == []):
                removeList.append(key)
            elif (isinstance(dct[key], SkeleYaml)):
                dct[key] = dct[key].toDict()
            elif (isinstance(dct[key], list)):
                dctList = []
                for element in dct[key]:
                    dctList.append(element.toDict() if isinstance(element, SkeleYaml) else element)
                dct[key] = dctList

        for key in removeList:
            del dct[key]

        return dct



from .systems.parsing import skeleParser
from .systems.generators import yaml
from .systems.execution import executor, docker, commandBuilder
import sys

# The main function for Skelebot CLI where the config is loaded, arguments are parsed, and commands are executed
def main():

    # Parse the env manually in order to read the correct yaml configuration
    args = sys.argv
    env = None
    for index in range(len(args)-1):
        if (args[index] == "-e") or (args[index] == "--env"):
            env = args[index+1]

    config = yaml.loadConfig(env)
    sbParser = skeleParser.SkeleParser(config, env)
    status = executor.execute(config, sbParser)



from ..objects.component import *
from ..systems.execution import docker
from ..systems.generators import dockerfile, dockerignore

# This component provides the ability to spin up Jupyter in Docker for any project
class Prime(Component):
    activation = Activation.PROJECT
    commands = ["prime"]

    # Parser for the command that primes the docker image for deployment or subsequent job execution
    def addParsers(self, subparsers):
        helpMessage = "Generate Dockerfile and .dockerignore and build the docker image"
        parser = subparsers.add_parser("prime", help=helpMessage)
        return subparsers

    # Generate the Dockerfile and dockerignore and build the docker image
    def execute(self, config, args):

        return docker.build(config)



from unittest import TestCase
from unittest import mock

import skelebot as sb
import os

class TestParser(TestCase):

    def test_parseArgs(self):
        arg1 = sb.objects.arg.Arg(name="start")
        arg2 = sb.objects.arg.Arg(name="end", choices=["now", "never"])
        param1 = sb.objects.param.Param(name="days", alt="d", default=10)
        param2 = sb.objects.param.Param(name="env", alt="e", default="local", choices=["local", "prod"])
        param3 = sb.objects.param.Param(name="method")
        param4 = sb.objects.param.Param(name="simple")
        topParam = sb.objects.param.Param(name="log", alt="l", default="info", choices=["debug", "info", "warn", "error"])
        job = sb.objects.job.Job(name="test", mode="d", source="test.py", help="TEST", args=[arg1, arg2], params=[param1, param2, param3, param4])
        config = sb.objects.config.Config(name="test-project", description="A test project", version="0.1.0", jobs=[job], params=[topParam])
        sbParser = sb.systems.parsing.skeleParser.SkeleParser(config, "test")
        args = sbParser.parseArgs(["test", "2019", "never", "-d", "20", "--env", "prod", "-l", "debug", "--method", "tune"])

        argKeys = list(vars(args).keys())
        self.assertEqual(args.job, "test")
        self.assertEqual(args.start, "2019")
        self.assertEqual(args.end, "never")
        self.assertEqual(args.days, "20")
        self.assertEqual(args.env, "prod")
        self.assertEqual(args.log, "debug")
        self.assertEqual(args.method, "tune")
        self.assertEqual(args.simple, None)

    def test_parseArgs_non_skelebot_scaffold(self):
        config = sb.objects.config.Config()
        sbParser = sb.systems.parsing.skeleParser.SkeleParser(config, "test")
        args = sbParser.parseArgs(["scaffold"])

        argKeys = list(vars(args).keys())
        self.assertEqual(args.job, "scaffold")
        self.assertEqual(args.existing, False)

    def test_description(self):
        config = sb.objects.config.Config(name="test-project", description="A test project", version="0.1.0")
        sbParser = sb.systems.parsing.skeleParser.SkeleParser(config, "test")
        description = sbParser.desc

        expectedDescription = """
\033[1mTest Project\033[0m
A test project
-----------------------------------
Version: 0.1.0
Environment: test
Skelebot Version: 1.0.1
-----------------------------------"""

        self.assertEqual(description, expectedDescription)

if __name__ == '__main__':
    unittest.main()



from ..scaffolding.scaffolder import scaffold
from .commandBuilder import build as buildCommand
from .docker import build as buildDocker
from .docker import run as runDocker
import argparse
import sys
import os

def execute(config, sbParser, args=sys.argv[1:]):
    for command in getCommands(args):
        print("=o=o= EXECUTING COMMAND: {} =o=o=".format(" ".join(command).upper()))
        args = sbParser.parseArgs(command)

        if (args.job == None):
            sbParser.showHelp()
        elif (args.job == "scaffold"):
            scaffold(args.existing)
        else:
            job = getJob(config, args)

            if (job is not None):
                executeJob(config, args, job)
            else:
                executeComponent(config, args)

def getCommands(args):
    commands = []
    command = []
    for arg in args:
        if arg == "+":
            commands.append(command)
            command = []
        else:
            command.append(arg)
    commands.append(command)

    return commands

def getJob(config, args):
    job = None
    for configJob in config.jobs:
        if args.job == configJob.name:
            job = configJob

    return job

def executeJob(config, args, job):
        command = buildCommand(config, job, args, args.native)
        if (args.native):
            os.system(command)
        else:
            if (args.skip_build == False):
                buildDocker(config)
            runDocker(config, command, job.mode, config.ports, job.mappings, job.name)

def executeComponent(config, args):
    for component in config.components:
        if (args.job in component.commands):
            component.execute(config, args)



from setuptools import setup, find_packages

VERSION = '0.0.0'
with open('VERSION', 'r') as version:
    VERSION = version.read().replace("\n", "")

setup(
    name="skelebot",
    version=VERSION,
    description="ML Build Tool",
    author="Sean Shookman",
    author_email="sshookman@cars.com",
    packages=find_packages(),
    zip_safe=False,
    setup_requires=["pytest-runner"],
    tests_require=[
        "pyyaml",
        "artifactory",
        "coverage",
        "pytest"
    ],
    install_requires=[
        "pyyaml==5.1.2",
        "artifactory==0.1.17",
        "requests==2.22.0",
        "coverage",
        "pytest"
    ],
    entry_points={
        'console_scripts': [
            'skelebot = skelebot:main',
        ],
    }
)



from unittest import TestCase
from unittest import mock

import skelebot as sb
import argparse
import os
import shutil

class TestArtifactory(TestCase):
    artifcatory = None

    def setUp(self):
        artifact = sb.components.artifactory.Artifact("test", "test.pkl")
        self.artifactory = sb.components.artifactory.Artifactory([artifact], "artifactory.test.com", "ml", "test")

    def test_addParsers(self):
        parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter)
        subparsers = parser.add_subparsers(dest="job")
        subparsers = self.artifactory.addParsers(subparsers)

        self.assertNotEqual(subparsers.choices["push"], None)
        self.assertNotEqual(subparsers.choices["pull"], None)

    @mock.patch('os.rename')
    @mock.patch('artifactory.ArtifactoryPath')
    def test_execute_push_conflict(self, mock_artifactory, mock_rename):
        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="push", force=False, artifact='test', user='sean', token='abc123')

        expectedException = """This artifact version has already been pushed.
Please bump the version before pushing (skelebot bump) or force push (-f)."""

        try:
            self.artifactory.execute(config, args)
            self.fail("Exception Not Thrown")
        except Exception as exc:
            self.assertEqual(str(exc), expectedException)


    @mock.patch('shutil.copyfile')
    @mock.patch('os.remove')
    @mock.patch('artifactory.ArtifactoryPath')
    def test_execute_push(self, mock_artifactory, mock_remove, mock_copy):
        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="push", force=True, artifact='test', user='sean', token='abc123')

        self.artifactory.execute(config, args)

        mock_artifactory.assert_called_with("artifactory.test.com/ml/test/test_v1.0.0.pkl", auth=('sean', 'abc123'))
        mock_copy.assert_called_with("test.pkl", "test_v1.0.0.pkl")
        mock_remove.assert_called_with("test_v1.0.0.pkl")

    @mock.patch('builtins.open')
    @mock.patch('artifactory.ArtifactoryPath')
    def test_execute_pull(self, mock_artifactory, mock_open):
        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="pull", version='0.1.0', artifact='test', user='sean', token='abc123')

        self.artifactory.execute(config, args)

        mock_artifactory.assert_called_with("artifactory.test.com/ml/test/test_v0.1.0.pkl", auth=('sean', 'abc123'))
        mock_open.assert_called_with("test_v0.1.0.pkl", "wb")

if __name__ == '__main__':
    unittest.main()



from unittest import TestCase
from unittest import mock

import skelebot as sb
import argparse
import os

class TestJupyter(TestCase):

    def test_addParsers(self):
        jupyter = sb.components.jupyter.Jupyter(port=1127, folder="notebooks/")

        parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter)
        subparsers = parser.add_subparsers(dest="job")
        subparsers = jupyter.addParsers(subparsers)

        self.assertNotEqual(subparsers.choices["jupyter"], None)

    @mock.patch('skelebot.components.jupyter.docker')
    def test_execute_R(self, mock_docker):
        mock_docker.build.return_value = 0
        config = sb.objects.config.Config(language="R")
        args = argparse.Namespace()

        jupyter = sb.components.jupyter.Jupyter(port=1127, folder="notebooks/")
        jupyter.execute(config, args)

        expectedCommand = "jupyter notebook --ip=0.0.0.0 --port=8888 --allow-root --notebook-dir=notebooks/"

        mock_docker.build.assert_called_with(config)
        mock_docker.run.assert_called_with(config, expectedCommand, "i", ["1127:8888"], ".", "jupyter")

    @mock.patch('skelebot.components.jupyter.docker')
    def test_execute_Python(self, mock_docker):
        mock_docker.build.return_value = 0
        config = sb.objects.config.Config(language="Python")
        args = argparse.Namespace()

        jupyter = sb.components.jupyter.Jupyter(port=1127, folder="notebooks/")
        jupyter.execute(config, args)

        expectedCommand = "jupyter notebook --ip=0.0.0.0 --port=8888 --allow-root --notebook-dir=notebooks/"

        mock_docker.build.assert_called_with(config)
        mock_docker.run.assert_called_with(config, expectedCommand, "i", ["1127:8888"], ".", "jupyter")

if __name__ == '__main__':
    unittest.main()
